@@ -41,16 +41,16 @@ unknown to the include path @INC, it can be including using the following code:
 
 =item perform_methodsv2_testing()
 
-    perform_methodsv2_testing( $href_subtests, $single_scenario, $disabled_scenarios );
+    perform_methodsv2_testing( $href_subtests, $selected_scenarios, $disabled_scenarios );
 
 This method loads unit test data (test scenarios) and, after some data checks and if the test scenario is testable,
 it runs all external L<MethodsV2|Zonemaster::Engine::Test::TestMethodsV2> methods and checks for the presence (or absence) of
 specific nameservers data for each specified test scenario.
 
-If C<$single_scenario> has been set in the call to the name of a scenario then only that
-scenario will be run, and it will always be run even if it has been set as not testable.
+If C<$selected_scenarios> has been set in the call to a comma separated list of names (or a single name), then only those
+scenarios will be run, and they will always be run even if they has been set as not testable.
 
-If C<$disabled_scenario> has been set in the call to the name of a scenario or to a
+If C<$disabled_scenarios> has been set in the call to the name of a scenario or to a
 comma separated list of scenarios then that or those scenarios will be
 temporarily disabled.
 
@@ -65,7 +65,7 @@ a boolean (testable), 1 or 0
 a string (zone name)
 
 =item *
-an array of strings (expected parent nameserver IPs), which could be empty, or undef
+an array of strings (expected parent nameserver expressions), which could be empty, or undef
 
 =item *
 an array of strings (expected delegation nameserver expressions), which could be empty, or undef
@@ -83,16 +83,16 @@ as the data for the --ns option in I<zonemaster-cli>.
 
 =item perform_testcase_testing()
 
-    perform_testcase_testing( $test_case, $test_module, $aref_alltags, $href_subtests, $single_scenario, $disabled_scenarios );
+    perform_testcase_testing( $test_case, $test_module, $aref_alltags, $href_subtests, $selected_scenarios, $disabled_scenarios );
 
 This method loads unit test data (test case name, test module name, array of all message tags and test scenarios) and,
 after some data checks and if the test scenario is testable, it runs the specified test case and checks for the presence
 (or absence) of specific message tags for each specified test scenario.
 
-If C<$single_scenario> has been set in the call to the name of a scenario then only that
-scenario will be run, and it will always be run even if it has been set as not testable.
+If C<$selected_scenarios> has been set in the call to a comma separated list of names of scenarios (or a single name) then only those
+scenarios will be run, and they will always be run even if they have been set as not testable.
 
-If C<$disabled_scenario> has been set in the call to the name of a scenario or to a
+If C<$disabled_scenarios> has been set in the call to the name of a scenario or to a
 comma separated list of scenarios then that or those scenarios will be
 temporarily disabled.
 
@@ -142,14 +142,6 @@ has the format "keytag,algorithm,type,digest". Those two expressions have the sa
 
 =over
 
-=item _check_ip_addresses()
-
-    _check_ip_addresses( $scenario_name, @ip_addresses );
-
-Helper method that checks if the given ip address(es) are valid.
-
-Takes a string (scenario name) and a reference to an array of strings (IP addresses).
-
 =item _check_ns_expressions()
 
     _check_ns_expressions( $scenario_name, @ns_expressions );
@@ -170,17 +162,6 @@ Takes a string (scenario name) and a reference to an array of strings (delegatio
 
 =cut
 
-sub _check_ip_addresses {
-    my ( $scenario, $ip_addresses ) = @_;
-
-    return if ! defined $ip_addresses;
-
-    foreach my $ip ( @{ $ip_addresses } ) {
-        croak "Scenario $scenario: IP address '$ip' is not valid"
-            unless validate_ipv4( $ip ) or validate_ipv6( $ip );
-    }
-}
-    
 sub _check_ns_expressions {
     my ( $scenario, $ns_expressions ) = @_;
 
@@ -210,20 +191,24 @@ sub _check_ds_expressions {
 }
 
 sub perform_methodsv2_testing {
-    my ( $href_subtests, $single_scenario, $disabled_scenarios ) = @_;
+    my ( $href_subtests, $selected_scenarios, $disabled_scenarios ) = @_;
     my %subtests = %$href_subtests;
 
-    $single_scenario = uc( $single_scenario ) if $single_scenario;
+    my @selected_scenarios = map {uc} split(/, */, $selected_scenarios) if $selected_scenarios;
     my @disabled_scenarios = map {uc} split(/, */, $disabled_scenarios) if $disabled_scenarios;
     
     my @untested_scenarios = ();
 
-    if ( $single_scenario and not grep /^$single_scenario$/, keys %subtests ) {
-        croak "Scenario $single_scenario does not exist";
+    if ( $selected_scenarios ) {
+        foreach my $scen (@selected_scenarios) {
+            unless ( exists $subtests{$scen} ) {
+                croak "Scenario $scen does not exist";
+            }
+        }
     }
 
     for my $scenario ( sort ( keys %subtests ) ) {
-        next if $single_scenario and $scenario ne $single_scenario;
+        next if $selected_scenarios and not grep /^$scenario$/, @selected_scenarios;
         if ( @disabled_scenarios and grep /^$scenario$/, @disabled_scenarios ) {
             push @untested_scenarios, $scenario;
             next;
@@ -238,7 +223,7 @@ sub perform_methodsv2_testing {
                 "Correct format is: { SCENARIO_NAME => [" .
                 "testable " .
                 "zone_name, " .
-                "[ EXPECTED_PARENT_IP ], " .
+                "[ EXPECTED_PARENT_NS ], " .
                 "[ EXPECTED_DEL_NS ], " .
                 "[ EXPECTED_ZONE_NS ], " .
                 "[ UNDELEGATED_NS ], " .
@@ -247,7 +232,7 @@ sub perform_methodsv2_testing {
 
         my ( $testable,
              $zone_name,
-             $expected_parent_ip,
+             $expected_parent_ns,
              $expected_del_ns,
              $expected_zone_ns,
              $undelegated_ns,
@@ -261,7 +246,7 @@ sub perform_methodsv2_testing {
             croak "Scenario $scenario: Value of testable must be 0 or 1";
         }
 
-        $testable = 1 if $single_scenario and $scenario eq $single_scenario;
+        $testable = 1 if $selected_scenarios and grep /^$scenario$/, @selected_scenarios;
 
         if ( ref( $zone_name ) ne '' ) {
             croak "Scenario $scenario: Type of zone name must not be a reference";
@@ -271,7 +256,7 @@ sub perform_methodsv2_testing {
             croak "Scenario $scenario: Zone name '$zone_name' is not valid";
         }
 
-        if ( defined( $expected_parent_ip ) and ref( $expected_parent_ip ) ne 'ARRAY' ) {
+        if ( defined( $expected_parent_ns ) and ref( $expected_parent_ns ) ne 'ARRAY' ) {
             croak "Scenario $scenario: Incorrect reference type of expected parent IPs. Expected: ARRAY";
         }
 
@@ -287,7 +272,7 @@ sub perform_methodsv2_testing {
             croak "Scenario $scenario: Incorrect reference type of undelegated name servers expressions. Expected: ARRAY";
         }
 
-        _check_ip_addresses( $scenario, $expected_parent_ip );
+        _check_ns_expressions( $scenario, $expected_parent_ns );
         _check_ns_expressions( $scenario, $expected_del_ns );
         _check_ns_expressions( $scenario, $expected_zone_ns );
         _check_ns_expressions( $scenario, $undelegated_ns );
@@ -306,21 +291,23 @@ sub perform_methodsv2_testing {
                     push @{ $undel_ns{$ns} }, $ip if $ip;
                 }
 
-                Zonemaster::Engine->add_fake_delegation( $zone_name => \%undel_ns, fill_in_empty_oob_glue => 0 );
+                # Use default value of "fill_in_empty_oob_glue".
+                Zonemaster::Engine->add_fake_delegation( $zone_name => \%undel_ns, fill_in_empty_oob_glue => 1 );
             }
 
-            # Method: get_parent_ns_ips()
-            my $method = 'get_parent_ns_ips';
+            # Method: get_parent_ns_names_and_ips()
+            my $method = 'get_parent_ns_names_and_ips';
             subtest $method => sub {
                 my $res = Zonemaster::Engine::TestMethodsV2->$method( Zonemaster::Engine->zone( $zone_name ) );
-                if ( defined $expected_parent_ip ) {
+                if ( defined $expected_parent_ns ) {
                     ok( defined $res, "Result is defined" ) or diag "Unexpected undefined result";
-                    foreach my $expected_ip ( @{ $expected_parent_ip } ) {
-                        ok( grep( /^$expected_ip$/, uniq map { $_->address->short } @{ $res } ), "Name server IP '$expected_ip' is present" )
-                            or diag "Expected but missing: $expected_ip";
+                    foreach my $expected_ns ( @{ $expected_parent_ns } ) {
+                        ok( grep( /^$expected_ns$/, @{ $res } ), "Name server '$expected_ns' is present" )
+                            or diag "Expected but missing: $expected_ns";
                     }
-                    ok( scalar @{ $res } == scalar @{ $expected_parent_ip }, "Number of name server IPs in both arrays match" )
-                        or diag "Number of name server IPs in both arrays does not match (found ". scalar @{ $res } . ", expected " . @{ $expected_parent_ip } . ")";
+                    ok( scalar @{ $res } == scalar @{ $expected_parent_ns }, "Number of name servers in both arrays match" )
+                        or diag "Number of name servers in both arrays does not match (found ". scalar @{ $res } . ", expected " . @{ $expected_parent_ns } . ")"
+                        or diag "Got:", explain [ map { "$_" } @$res ];
                 }
                 else {
                     ok( ! defined $res, "Result is undefined" ) or diag "Unexpected defined result";
@@ -427,16 +414,20 @@ sub perform_methodsv2_testing {
 
 
 sub perform_testcase_testing {
-    my ( $test_case, $test_module, $aref_alltags, $href_subtests, $single_scenario, $disabled_scenarios ) = @_;
+    my ( $test_case, $test_module, $aref_alltags, $href_subtests, $selected_scenarios, $disabled_scenarios ) = @_;
     my %subtests = %$href_subtests;
 
-    $single_scenario = uc( $single_scenario ) if $single_scenario;
+    my @selected_scenarios = map {uc} split(/, */, $selected_scenarios) if $selected_scenarios;
     my @disabled_scenarios = map {uc} split(/, */, $disabled_scenarios) if $disabled_scenarios;
 
     my @untested_scenarios = ();
 
-    if ( $single_scenario and not grep /^$single_scenario$/, keys %subtests ) {
-        croak "Scenario $single_scenario does not exist";
+    if ( $selected_scenarios ) {
+        foreach my $scen (@selected_scenarios) {
+            unless ( exists $subtests{$scen} ) {
+                croak "Scenario $scen does not exist";
+            }
+        }
     }
 
     if ( ref( $aref_alltags ) ne 'ARRAY' ) {
@@ -448,7 +439,7 @@ sub perform_testcase_testing {
     }
 
     for my $scenario ( sort ( keys %subtests ) ) {
-        next if $single_scenario and $scenario ne $single_scenario;
+        next if $selected_scenarios and not grep /^$scenario$/,  @selected_scenarios;
         if ( @disabled_scenarios and grep /^$scenario$/, @disabled_scenarios ) {
             push @untested_scenarios, $scenario;
             next;
@@ -486,7 +477,7 @@ sub perform_testcase_testing {
             croak "Scenario $scenario: Value of testable must be 0 or 1";
         }
 
-        $testable = 1 if $single_scenario and $scenario eq $single_scenario;
+        $testable = 1 if $selected_scenarios and grep /^$scenario$/, @selected_scenarios;
 
         if ( ref( $zone_name ) ne '' ) {
             croak "Scenario $scenario: Type of zone name must not be a reference";
@@ -575,7 +566,8 @@ sub perform_testcase_testing {
                     push @{ $undel_ns{$ns} }, $ip if $ip;
                 }
 
-                Zonemaster::Engine->add_fake_delegation( $zone_name => \%undel_ns, fill_in_empty_oob_glue => 0 );
+                # Use default value of "fill_in_empty_oob_glue".
+                Zonemaster::Engine->add_fake_delegation( $zone_name => \%undel_ns, fill_in_empty_oob_glue => 1 );
             }
 
             if ( @$undelegated_ds ) {
